@@ -55,7 +55,7 @@ extern DECLSPEC void libcec_close(libcec_connection_t connection);
 extern DECLSPEC void libcec_clear_configuration(CEC_NAMESPACE libcec_configuration* configuration);
 #if CEC_LIB_VERSION_MAJOR >= 5
 extern DECLSPEC int libcec_set_callbacks(libcec_connection_t connection, CEC_NAMESPACE ICECCallbacks* callbacks, void* cbParam);
-extern DECLSPEC int libcec_disabled_callbacks(libcec_connection_t connection);
+extern DECLSPEC int libcec_disable_callbacks(libcec_connection_t connection);
 #else
 extern DECLSPEC int libcec_enable_callbacks(libcec_connection_t connection, void* cbParam, CEC_NAMESPACE ICECCallbacks* callbacks);
 #endif
@@ -116,6 +116,8 @@ extern DECLSPEC uint8_t libcec_audio_toggle_mute(libcec_connection_t connection)
 extern DECLSPEC uint8_t libcec_audio_mute(libcec_connection_t connection);
 extern DECLSPEC uint8_t libcec_audio_unmute(libcec_connection_t connection);
 extern DECLSPEC uint8_t libcec_audio_get_status(libcec_connection_t connection);
+extern DECLSPEC int libcec_system_audio_mode(libcec_connection_t connection, int bEnable);
+extern DECLSPEC uint8_t libcec_system_audio_mode_get_status(libcec_connection_t connection);
 extern DECLSPEC int8_t libcec_detect_adapters(libcec_connection_t connection, CEC_NAMESPACE cec_adapter_descriptor* deviceList, uint8_t iBufSize, const char* strDevicePath, int bQuickScan);
 #ifdef SWIG
 %cstring_bounded_output(char* buf, 50);
