@@ -532,7 +532,7 @@ const char *CLibCEC::GetLibInfo(void)
 #define FEATURES "'P8 USB' 'P8 USB detect'"
 #if defined(_WIN64)
 #define HOST_TYPE "Windows (x64)"
-#elseif defined(_M_ARM64)
+#elif defined(_M_ARM64)
 #define HOST_TYPE "Windows (ARM64)"
 #else
 #define HOST_TYPE "Windows (x86)"
@@ -582,6 +582,11 @@ uint8_t CLibCEC::AudioStatus(void)
   return m_client ? m_client->AudioStatus() : (uint8_t)CEC_AUDIO_VOLUME_STATUS_UNKNOWN;
 }
 
+uint8_t CLibCEC::SystemAudioModeStatus(void)
+{
+  return m_client ? m_client->SystemAudioModeStatus() : (uint8_t)CEC_SYSTEM_AUDIO_STATUS_UNKNOWN;
+}
+
 int8_t CLibCEC::DetectAdapters(cec_adapter_descriptor *deviceList, uint8_t iBufSize, const char *strDevicePath /* = nullptr */, bool bQuickScan /* = false */)
 {
   int8_t iAdaptersFound = CAdapterFactory(this).DetectAdapters(deviceList, iBufSize, strDevicePath);
