@@ -42,7 +42,7 @@
 
 #include "CECTypeUtils.h"
 #include "LibCEC.h"
-#include <p8-platform/util/buffer.h>
+#include "p8-platform/util/buffer.h"
 
 using namespace CEC;
 using namespace P8PLATFORM;
@@ -172,8 +172,9 @@ uint16_t CExynosCECAdapterCommunication::GetPhysicalAddress(void)
 }
 
 
-cec_logical_addresses CExynosCECAdapterCommunication::GetLogicalAddresses(void)
+cec_logical_addresses CExynosCECAdapterCommunication::GetLogicalAddresses(void) const
 {
+  CLockObject lock(m_mutex);
   return m_logicalAddresses;
 }
 
