@@ -76,6 +76,7 @@ CUSBCECAdapterCommunication::CUSBCECAdapterCommunication(IAdapterCommunicationCa
     m_commands(NULL),
     m_adapterMessageQueue(NULL)
 {
+  memset(&m_stats, 0, sizeof(struct cec_adapter_stats));
   m_logicalAddresses.Clear();
   for (unsigned int iPtr = CECDEVICE_TV; iPtr < CECDEVICE_BROADCAST; iPtr++)
     m_bWaitingForAck[iPtr] = false;
@@ -269,7 +270,6 @@ cec_adapter_message_state CUSBCECAdapterCommunication::Write(const cec_command &
 
 void *CUSBCECAdapterCommunication::Process(void)
 {
-  CCECAdapterMessage msg;
   LIB_CEC->AddLog(CEC_LOG_DEBUG, "communication thread started");
 
   while (!IsStopped())
@@ -380,7 +380,7 @@ bool CUSBCECAdapterCommunication::WriteToDevice(CCECAdapterMessage *message)
   }
 
   /* write the message */
-  if (m_port->Write(message->packet.data, message->Size()) != (ssize_t) message->Size())
+  if (m_port->Write(message->m_tx_data, message->m_tx_len) != (ssize_t)message->m_tx_len)
   {
     LIB_CEC->AddLog(CEC_LOG_DEBUG, "error writing command '%s' to serial port '%s': %s", CCECAdapterMessage::ToString(message->Message()), m_port->GetName().c_str(), m_port->GetError().c_str());
     message->state = ADAPTER_MESSAGE_STATE_ERROR;
@@ -517,7 +517,11 @@ bool CUSBCECAdapterCommunication::CheckAdapter(uint32_t iTimeoutMs /* = CEC_DEFA
 bool CUSBCECAdapterCommunication::IsOpen(void)
 {
   /* thread is not being stopped, the port is open and the thread is running */
-  return !IsStopped() && m_port->IsOpen() && IsRunning();
+  bool stopped, opened, running;
+  stopped = IsStopped();
+  opened = m_port->IsOpen();
+  running = IsRunning();
+  return !stopped && opened && running;
 }
 
 std::string CUSBCECAdapterCommunication::GetError(void) const
@@ -635,16 +639,25 @@ void CUSBCECAdapterCommunication::SetActiveSource(bool bSetTo, bool bClientUnreg
     m_commands->SetActiveSource(bSetTo, bClientUnregistered);
 }
 
+#if CEC_LIB_VERSION_MAJOR >= 5
+bool CUSBCECAdapterCommunication::GetStats(struct cec_adapter_stats* stats)
+{
+  CLockObject lock(m_statsMutex);
+  memcpy(stats, &m_stats, sizeof(struct cec_adapter_stats));
+  return true;
+}
+#endif
+
 bool CUSBCECAdapterCommunication::IsRunningLatestFirmware(void)
 {
   return GetFirmwareBuildDate() >= CEC_LATEST_ADAPTER_FW_DATE &&
       GetFirmwareVersion() >= CEC_LATEST_ADAPTER_FW_VERSION;
 }
 
-bool CUSBCECAdapterCommunication::PersistConfiguration(const libcec_configuration &configuration)
+bool CUSBCECAdapterCommunication::SaveConfiguration(const libcec_configuration &configuration)
 {
   return IsOpen() ?
-      m_commands->PersistConfiguration(configuration) && m_eepromWriteThread->Write() :
+      m_commands->SaveConfiguration(configuration) && m_eepromWriteThread->Write() :
       false;
 }
 
@@ -717,6 +730,36 @@ uint16_t CUSBCECAdapterCommunication::GetPhysicalAddress(void)
   return iPA;
 }
 
+void CUSBCECAdapterCommunication::OnRxSuccess(void)
+{
+  CLockObject lock(m_statsMutex);
+  ++m_stats.rx_total;
+}
+
+void CUSBCECAdapterCommunication::OnRxError(void)
+{
+  CLockObject lock(m_statsMutex);
+  ++m_stats.rx_error;
+}
+
+void CUSBCECAdapterCommunication::OnTxAck(void)
+{
+  CLockObject lock(m_statsMutex);
+  ++m_stats.tx_ack;
+}
+
+void CUSBCECAdapterCommunication::OnTxNack(void)
+{
+  CLockObject lock(m_statsMutex);
+  ++m_stats.tx_nack;
+}
+
+void CUSBCECAdapterCommunication::OnTxError(void)
+{
+  CLockObject lock(m_statsMutex);
+  ++m_stats.tx_error;
+}
+
 void *CAdapterPingThread::Process(void)
 {
   while (!IsStopped())
