@@ -53,14 +53,17 @@ namespace CEC
     uint8_t VolumeDown(const cec_logical_address source, bool bSendRelease = true);
     uint8_t MuteAudio(const cec_logical_address source);
     uint8_t GetAudioStatus(const cec_logical_address initiator, bool bUpdate = false);
-    bool EnableAudio(CCECBusDevice* device = nullptr);
+    uint8_t GetSystemAudioModeStatus(const cec_logical_address initiator, bool bUpdate = false);
+    bool AudioEnable(CCECBusDevice* device, bool bEnable = true);
 
-    bool TransmitActiveSource(void) { return false; }
+    bool TransmitActiveSource(bool bIsReply) { (void)bIsReply; return false; }
 
   protected:
     bool RequestAudioStatus(const cec_logical_address initiator, bool bWaitForResponse = true);
+    bool RequestSystemAudioModeStatus(const cec_logical_address initiator, bool bWaitForResponse = true);
 
-    cec_system_audio_status m_systemAudioStatus;
+    int64_t                 m_iLastAudioStatusUpdate;
+    uint8_t                 m_systemAudioModeStatus;
     uint8_t                 m_audioStatus;
   };
 }
